from unittest2 import TestCase
from aux.logger import LogController


class LoggingTest(TestCase):

    def xtest_start_logging(self):
        pass
        #setting up log controller
        #logcontroller = LogController(config)

        #test write formats

        # log.info("This is a info message.")
        # log.debug("This is a debug message.")
        # log.error("This is an error message.")
        # log.warning("This is a warning message.")
        # log.critical("This is a critical message.")
