# -*- coding: utf-8 -*-
# from aux.system.base import BaseSystem
from linux import LinuxDevice


#TODO: obviously this is just thrown together

# class Device(BaseSystem):
#     def __init__(self, hostname):
#         super(Service, self).__init__(hostname)
#         self.hostname = hostname
